package com.voter.verification.dao;

import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;

import com.voter.verification.model.Voter;

/**
 * Voter Repository Implementation class to operate the CRUD operations.
 * @author rjosula
 *
 */
@Repository
public class VoterDAOImpl implements VoterDAO {

	@Autowired
	private MongoTemplate mongoTemplate;
	
	private Logger logger = LogManager.getLogger(VoterDAOImpl.class);
	
	/**
	 * Save or update the Voter information based on the personId criteria
	 * If person Id exists it will update else will create a new record.
	 */
	@Override
	public Voter saveOrUpdateVoterInfo(Voter voterObj) {
		Voter voterSavedObj = null;
		Query query = new Query();
		query.addCriteria(Criteria.where("personId").is(Integer.valueOf(voterObj.getPersonId())));
		try {
	        Update update = new Update();
	        update.set("personId" , voterObj.getPersonId());
	        update.set("personAge" , voterObj.getPersonAge());
	        update.set("personName", voterObj.getPersonName());
	        update.set("personAddress", voterObj.getPersonAddress());
	        update.set("state", voterObj.getState());
	        
	        voterSavedObj = mongoTemplate.findAndModify(query, update, Voter.class);
	        if(voterSavedObj == null ) {
	        	voterSavedObj = mongoTemplate.save(voterObj);
	        }
	        logger.info("Updated the information ... " + voterSavedObj.toString());
			
		} catch(Exception ex) {
			logger.error("Error while saving the voter information", ex);
		}
		return voterSavedObj;
	}

	@Override
	public List<Voter> getListOfVoterInfo(Integer id) {
		return mongoTemplate.find(new Query().addCriteria(Criteria.where("personId").is(id)), Voter.class);
	}

}
