package com.voter.verification.service;

import java.io.StringWriter;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.stereotype.Component;

import com.voter.verification.dao.VoterDAO;
import com.voter.verification.model.Voter;
import com.voter.verification.model.VoterInformation;
import com.voter.verification.vo.VoterVO;

/**
 * Service activator to consume & apply business logic to transform from JSON to
 * XML
 * 
 * @author rjosula
 *
 */
@Component
public class VoterServiceActivator {
	
	@Autowired
	private VoterDAO voterDAO;
	
	private Logger logger = LogManager.getLogger(VoterServiceActivator.class);

	/**
	 * Service activator method to receive the input message
	 * 
	 * @param inputMesg
	 * @return
	 */
	public Message<VoterInformation> mapPersonToVoter(Message<VoterVO> personMsg) {
		logger.info("Inside service activator... " + personMsg.getPayload());
		VoterVO voterInput = personMsg.getPayload();
		Voter voterObjSaved = null;
		boolean isPersonEligible = false;
		if (voterInput.getPersonAge() >= 18) {
			isPersonEligible = true;
		}
		
		Voter dbInput = new Voter();
		/**
		 * This is to copy properties from source voterInput to dbInput destination
		 */
		try {
			BeanUtils.copyProperties(dbInput, voterInput);
		} catch(Exception ex) {
			logger.error("Error occured while copying the properties ", ex);
		}
		
		/** Saving the information using DAO Layer*/
		voterObjSaved = voterDAO.saveOrUpdateVoterInfo(dbInput);
		logger.info("Voter information has been successfully saved or updated!...."+voterObjSaved.toString());
		
		VoterInformation voterInfo = new VoterInformation();
		voterInfo.setVoterId(dbInput.getPersonId());
		voterInfo.setVoterName(dbInput.getPersonName());
		voterInfo.setVoterAddress(dbInput.getPersonAddress());
		voterInfo.setVoterProvince(dbInput.getState());
		voterInfo.setEligible(isPersonEligible);
		voterInfo.setVoterAge(voterInput.getPersonAge());
		voterInfo.setReferenceId(voterInput.getReferenceId());
		voterInfo.setEmailId(voterInput.getEmailId());

		return MessageBuilder.withPayload(voterInfo).build();
	}

	/**
	 * Takes input as DomResult XML and converts to XML String.
	 * 
	 * @param inputJson
	 * @return
	 */
	public String voterXMLInput(Message<DOMResult> inputJson) {
		DOMResult payload = inputJson.getPayload();
		Transformer transformer;
		Message<String> msgOutput = null;
		try {
			transformer = TransformerFactory.newInstance().newTransformer();
			StringWriter writer = new StringWriter();
			StreamResult result = new StreamResult(writer);
			Source input = new DOMSource(payload.getNode());
			transformer.transform(input, result);
			msgOutput = MessageBuilder.createMessage(result.getWriter().toString(), inputJson.getHeaders());
		} catch (TransformerFactoryConfigurationError | TransformerException e) {
			e.printStackTrace();
		}
		return msgOutput.getPayload();
	}
	
}
